#
# Script that can be used to install the Green Hat jdbc provider into WebSphere
#
# Usage:
# wsadmin.bat -f setupJDBCProvider.py <install directory> [node name]
#

newline = java.lang.System.getProperty("line.separator")

import sys

if len(sys.argv) < 1 or len(sys.argv) > 2:
  sys.exit("\nUsage: wsadmin.bat -f setupJDBCProvider.py <install directory> [node name]\n\n" +
           "  <install directory> should include the driver jar files, and is required\n" +
           "  [node name] is the WebSphere node name to configure, and is optional\n"+
           "\n"+
           "  Please use forward slashes in the install directory path")

# Any input such as '\t' will have been mapped into a tab character by the time that
# we get the path, so we need to reverse that mapping. This is not perfect, as user
# strings including '\x' or '\\' will still be mapped incorrectly or may fail. If
# there are problems then it will help to use forward slashes ('/') in the path.
raw_input = sys.argv[0]
gh_install_path = ""
for ch in raw_input:
  if ch == '\a':
    gh_install_path = gh_install_path + '/a'
  elif ch == '\b':
    gh_install_path = gh_install_path + '/b'
  elif ch == '\f':
    gh_install_path = gh_install_path + '/f'
  elif ch == '\n':
    gh_install_path = gh_install_path + '/n'
  elif ch == '\r':
    gh_install_path = gh_install_path + '/r'
  elif ch == '\t':
    gh_install_path = gh_install_path + '/t'
  elif ch == '\v':
    gh_install_path = gh_install_path + '/v'
  elif ch == '\\':
    gh_install_path = gh_install_path + '/'
  else:
    gh_install_path = gh_install_path + ch

node = 0
nodes = AdminConfig.list("Node").split(newline)

if len(sys.argv) == 2:
  nodeName = sys.argv[1]
  for candidate in nodes:
    name = AdminConfig.showAttribute(nodes[0], "name")
    if name == nodeName:
      node = candidate
      break
      
  if not node:
    sys.exit("ERROR: Could not locate node " + nodeName)

else:
  if len(nodes) > 1:
    sys.exit("ERROR: There is more than one WebSphere node. Please supply a node name")
  
  node = nodes[0]
  nodeName = AdminConfig.showAttribute(node, "name")

print "\nInstalling Green Hat JDBC provider from " + gh_install_path + " onto node " + nodeName

required_libs = [
    'greenhat.jdbcdriver.jar',
    'jsqlparser15.jar'
]

for lib in required_libs:
  file = gh_install_path + '/' + lib
  try:
    fh = open(file, 'r')
    fh.close
  except IOError:
    sys.exit('ERROR: Failed to find required library at path: ' + file)

registration_xml = gh_install_path + '/registration.xml'
try:
  fh = open(registration_xml, 'r')
  fh.close
except IOError:
  print "\nWARNING: The Green Hat JDBC driver registration file was not found." \
  	+	"\nPlease set JVM property com.greenhat.registration to point to the " \
  	+	"registration file, or copy the sample registration.xml file to " \
  	+	registration_xml

print "\nSetting GH_DRIVER_PATH onto node " + nodeName
varName = "GH_DRIVER_PATH"
variableMap = AdminConfig.getid("/Node:"+nodeName+"/VariableMap:/")

found = 0
variables = AdminConfig.list("VariableSubstitutionEntry",variableMap).split(newline)
for variable in variables:
  name = AdminConfig.showAttribute(variable, "symbolicName")
  if name == varName:
    print "\tUpdating existing WebSphere variable " + varName
    AdminConfig.modify(variable,[["value", gh_install_path]])
    found = 1
    break
    
if not found:
  print "\tCreating new WebSphere variable " + varName
  AdminConfig.create("VariableSubstitutionEntry",variableMap,[["symbolicName",varName],["value",gh_install_path]])

print "\nSetting up JDBC provider"

providerName = "Green Hat JDBC Provider"
driverClassPath = ""
for lib in required_libs:
  driverClassPath += "${" + varName + "}/" + lib + ";"

properties = [
  ["name",providerName],
  ["implementationClassName","com.greenhat.jdbc.XADataSource"],
  ["classpath",driverClassPath],
  ["isolatedClassLoader","true"],
  ["xa","true"],
  ["description","Green Hat JDBC Provider. This provider enables database stubbing with IBM Rational Integration Tester."]
]

provider = 0
providers = AdminConfig.list("JDBCProvider",node).split(newline)
for candidate in providers:
  name = AdminConfig.showAttribute(candidate,"name")
  if name == providerName:
    provider = candidate
    break

if not provider:
  print "\tCreating new JDBC provider"
  properties.append(["providerType","User-defined JDBC Provider"])
  provider = AdminConfig.create("JDBCProvider",node,properties)
else:
  print "\tUpdating existing JDBC provider"
  AdminConfig.resetAttributes(provider,properties)

AdminConfig.save()
print "\nGreen Hat JDBC Provider installation completed successfully"
